@@ -27,8 +27,8 @@ import time
 from gettext import gettext as _
 
 import gi
-gi.require_versions({"Grl": "0.3"})
-from gi.repository import Gio, Grl, GLib, GObject, Tracker
+gi.require_versions({"Grl": "0.3", "Tracker": "3.0"})
+from gi.repository import Gio, Grl, Gtk, GLib, GObject, Tracker
 
 from gnomemusic.coresong import CoreSong
 import gnomemusic.utils as utils
@@ -68,7 +68,9 @@ class GrlTrackerPlaylists(GObject.GObject):
         self._notificationmanager = application.props.notificationmanager
         self._window = application.props.window
 
-        self._user_model_filter.set_filter_func(self._user_playlists_filter)
+        user_playlists_filter = Gtk.CustomFilter()
+        user_playlists_filter.set_filter_func(self._user_playlists_filter)
+        self._user_model_filter.set_filter(user_playlists_filter)
 
         self._fast_options = Grl.OperationOptions()
         self._fast_options.set_resolution_flags(
@@ -146,7 +148,9 @@ class GrlTrackerPlaylists(GObject.GObject):
         :param Playlist playlist: playlist
         """
         self._pls_todelete.append(playlist)
-        self._model_filter.set_filter_func(self._playlists_filter)
+        playlists_filter = Gtk.CustomFilter()
+        playlists_filter.set_filter_func(self._playlists_filter)
+        self._model_filter.set_filter(playlists_filter)
 
     def finish_playlist_deletion(self, playlist, deleted):
         """Removes playlist from the list of playlists to delete
@@ -154,11 +158,15 @@ class GrlTrackerPlaylists(GObject.GObject):
         :param Playlist playlist: playlist
         :param bool deleted: indicates if the playlist has been deleted
         """
+        playlists_filter = Gtk.CustomFilter()
+        playlists_filter.set_filter_func(self._playlists_filter)
+        user_playlists_filter = Gtk.CustomFilter()
+        user_playlists_filter.set_filter_func(self._playlists_filter)
+
         self._pls_todelete.remove(playlist)
         if deleted is False:
-            self._model_filter.set_filter_func(self._playlists_filter)
-            self._user_model_filter.set_filter_func(
-                self._user_playlists_filter)
+            self._model_filter.set_filter(playlists_filter)
+            self._user_model_filter.set_filter(user_playlists_filter)
             return
 
         def _delete_cb(conn, res, data):
@@ -173,7 +181,9 @@ class GrlTrackerPlaylists(GObject.GObject):
                         self._model.remove(idx)
                         break
 
-            self._model_filter.set_filter_func(self._playlists_filter)
+            playlists_filter = Gtk.CustomFilter()
+            playlists_filter.set_filter_func(self._playlists_filter)
+            self._model_filter.set_filter(playlists_filter)
             self._notificationmanager.pop_loading()
 
         self._notificationmanager.push_loading()
@@ -185,7 +195,7 @@ class GrlTrackerPlaylists(GObject.GObject):
             ?playlist a nmm:Playlist ;
                       a nfo:MediaList .
             FILTER (
-            tracker:id(?playlist) = %(playlist_id)s
+                ?playlist = <%(playlist_id)s>
             )
         }
         """.replace("\n", " ").strip() % {
@@ -282,6 +292,7 @@ class Playlist(GObject.GObject):
 
     count = GObject.Property(type=int, default=0)
     creation_date = GObject.Property(type=GLib.DateTime, default=None)
+    icon_name = GObject.Property(type=str, default="music-playlist-symbolic")
     is_smart = GObject.Property(type=bool, default=False)
     pl_id = GObject.Property(type=str, default=None)
     query = GObject.Property(type=str, default=None)
@@ -390,7 +401,7 @@ class Playlist(GObject.GObject):
         ORDER BY nfo:listPosition(?entry)
         """.replace('\n', ' ').strip() % {
             "media_type": int(Grl.MediaType.AUDIO),
-            "filter_clause": 'tracker:id(?playlist) = ' + self.props.pl_id,
+            "filter_clause": f"?playlist = <{self.props.pl_id}>",
             "location_filter": self._tracker_wrapper.location_filter(),
             "miner_fs_busname": self._tracker_wrapper.props.miner_fs_busname,
         }
@@ -477,7 +488,7 @@ class Playlist(GObject.GObject):
                 ?playlist nfo:hasMediaFileListEntry ?entry .
             }
             FILTER (
-                tracker:id(?playlist) = %(playlist_id)s
+                ?playlist = <%(playlist_id)s>
             )
         }
         """.replace("\n", " ").strip() % {
@@ -508,10 +519,12 @@ class Playlist(GObject.GObject):
         self._model.insert(position, coresong)
         self.props.count += 1
 
-    def finish_song_deletion(self, coresong):
+    def finish_song_deletion(self, coresong: CoreSong, position: int) -> None:
         """Removes a song from the playlist
 
         :param CoreSong coresong: song to remove
+        :param int position: Song position in the playlist, starts from
+        zero
         """
         def update_cb(conn, res):
             try:
@@ -554,8 +567,8 @@ class Playlist(GObject.GObject):
                                         ?removed_entry .
                             ?removed_entry nfo:listPosition ?removed_position .
                             FILTER (
-                                tracker:id(?playlist) = %(playlist_id)s &&
-                                tracker:id(?removed_entry) = %(entry_id)s
+                                ?playlist = <%(playlist_id)s> &&
+                                ?removed_entry = <%(entry_id)s>
                             )
                         }
                     }
@@ -572,7 +585,7 @@ class Playlist(GObject.GObject):
                               a nfo:MediaList ;
                                 nfo:entryCounter ?counter .
                     FILTER (
-                        tracker:id(?playlist) = %(playlist_id)s
+                        ?playlist = <%(playlist_id)s>
                     )
                 }
             };
@@ -585,8 +598,8 @@ class Playlist(GObject.GObject):
                           a nfo:MediaList ;
                             nfo:hasMediaFileListEntry ?entry .
                 FILTER (
-                    tracker:id(?playlist) = %(playlist_id)s &&
-                    tracker:id(?entry) = %(entry_id)s
+                    ?playlist = <%(playlist_id)s> &&
+                    ?entry = <%(entry_id)s>
                 )
             }
             """.replace("\n", " ").strip() % {
@@ -596,7 +609,6 @@ class Playlist(GObject.GObject):
 
             self._tracker.update_async(update_query, None, update_cb)
 
-        song_id = coresong.props.media.get_id()
         entry_query = """
         SELECT
             %(media_type)s AS ?type
@@ -605,31 +617,16 @@ class Playlist(GObject.GObject):
                 ?playlist a nmm:Playlist ;
                           a nfo:MediaList ;
                             nfo:hasMediaFileListEntry ?entry .
-                ?entry a nfo:MediaFileListEntry ;
-                         nfo:entryUrl ?url .
-                SERVICE <dbus:%(miner_fs_busname)s> {
-                    GRAPH tracker:Audio {
-                        SELECT
-                            ?song
-                            ?url
-                        WHERE {
-                            ?song a nmm:MusicPiece ;
-                                  nie:isStoredAs ?url .
-                            FILTER (
-                                %(filter_clause_song)s
-                            )
-                        }
-                    }
-                }
+                ?entry a nfo:MediaFileListEntry .
                 FILTER (
-                    %(filter_clause_pl)s
+                    %(filter_clause_pl)s &&
+                    nfo:listPosition(?entry) = %(position)s
                 )
             }
         """.replace("\n", " ").strip() % {
             "media_type": int(Grl.MediaType.AUDIO),
-            "filter_clause_song": "tracker:id(?song) = " + song_id,
-            "filter_clause_pl": "tracker:id(?playlist) = " + self.props.pl_id,
-            "miner_fs_busname": self._tracker_wrapper.props.miner_fs_busname
+            "position": position + 1,
+            "filter_clause_pl": f"?playlist = <{self.props.pl_id}>",
         }
 
         self._source.query(
@@ -705,8 +702,8 @@ class Playlist(GObject.GObject):
             } LIMIT 1
             """.replace("\n", " ").strip() % {
                 "media_type": int(Grl.MediaType.AUDIO),
-                "filter_clause_song": "tracker:id(?song) = " + media_id,
-                "filter_clause_pl": "tracker:id(?playlist) = " + pl_id,
+                "filter_clause_song": f"?song = <{media_id}>",
+                "filter_clause_pl": f"?playlist = <{pl_id}>",
                 "location_filter": self._tracker_wrapper.location_filter(),
                 "miner_fs_busname": miner_fs_busname,
             }
@@ -732,7 +729,7 @@ class Playlist(GObject.GObject):
                               a nfo:MediaList ;
                                 nfo:entryCounter ?counter .
                     FILTER (
-                        tracker:id(?playlist) = %(playlist_id)s
+                        ?playlist = <%(playlist_id)s>
                     )
                 }
             }
@@ -767,8 +764,8 @@ class Playlist(GObject.GObject):
                       a nfo:MediaList ;
                         nfo:hasMediaFileListEntry ?entry .
             FILTER (
-                tracker:id(?playlist) = %(playlist_id)s &&
-                tracker:id(?entry) = %(song_id)s
+                ?playlist = <%(playlist_id)s> &&
+                ?entry = <%(song_id)s>
             )
         }
         """.replace("\n", " ").strip()
@@ -895,6 +892,7 @@ class MostPlayed(SmartPlaylist):
         self.props.tag_text = "MOST_PLAYED"
         # TRANSLATORS: this is a playlist name
         self._title = _("Most Played")
+        self.props.icon_name = "audio-speakers-symbolic"
         self.props.query = """
         SELECT
             %(media_type)s AS ?type
@@ -948,6 +946,7 @@ class NeverPlayed(SmartPlaylist):
         self.props.tag_text = "NEVER_PLAYED"
         # TRANSLATORS: this is a playlist name
         self._title = _("Never Played")
+        self.props.icon_name = "dialog-question-symbolic"
         self.props.query = """
         SELECT
             %(media_type)s AS ?type
@@ -1000,6 +999,7 @@ class RecentlyPlayed(SmartPlaylist):
         self.props.tag_text = "RECENTLY_PLAYED"
         # TRANSLATORS: this is a playlist name
         self._title = _("Recently Played")
+        self.props.icon_name = "document-open-recent-symbolic"
 
         sparql_midnight_dateTime_format = "%Y-%m-%dT00:00:00Z"
         days_difference = 7
@@ -1021,30 +1021,46 @@ class RecentlyPlayed(SmartPlaylist):
             nie:usageCounter(?song) AS ?playCount
             ?tag AS ?favorite
         WHERE {
-            SERVICE <dbus:%(miner_fs_busname)s> {
-                GRAPH tracker:Audio {
-                    SELECT
-                        ?song
-                        nie:title(?song) AS ?title
-                        nie:isStoredAs(?song) AS ?url
-                        nmm:artistName(nmm:artist(?song)) AS ?artist
-                        nie:title(nmm:musicAlbum(?song)) AS ?album
-                        nfo:duration(?song) AS ?duration
-                        nmm:trackNumber(?song) AS ?trackNumber
-                        nmm:setNumber(nmm:musicAlbumDisc(?song))
-                            AS ?albumDiscNumber
-                    WHERE {
-                        ?song a nmm:MusicPiece .
-                        %(location_filter)s
+            {
+                SELECT
+                    ?song
+                    ?title
+                    ?url
+                    ?artist
+                    ?album
+                    ?duration
+                    ?trackNumber
+                    ?albumDiscNumber
+                    ?playCount
+                    ?tag
+                    ?lastPlayed
+                WHERE {
+                    SERVICE <dbus:%(miner_fs_busname)s> {
+                        GRAPH tracker:Audio {
+                            SELECT
+                                ?song
+                                nie:title(?song) AS ?title
+                                nie:isStoredAs(?song) AS ?url
+                                nmm:artistName(nmm:artist(?song)) AS ?artist
+                                nie:title(nmm:musicAlbum(?song)) AS ?album
+                                nfo:duration(?song) AS ?duration
+                                nmm:trackNumber(?song) AS ?trackNumber
+                                nmm:setNumber(nmm:musicAlbumDisc(?song))
+                                    AS ?albumDiscNumber
+                            WHERE {
+                                ?song a nmm:MusicPiece .
+                                %(location_filter)s
+                            }
+                        }
                     }
-                }
+                    ?song nie:contentAccessed ?lastPlayed ;
+                        nie:usageCounter ?playCount .
+                    OPTIONAL { ?song nao:hasTag ?tag .
+                               FILTER (?tag = nao:predefined-tag-favorite) }
+                } ORDER BY DESC(?lastPlayed) LIMIT 50
             }
-            ?song nie:contentAccessed ?lastPlayed .
-            FILTER ( ?lastPlayed > '%(compare_date)s'^^xsd:dateTime
-                     && EXISTS { ?song nie:usageCounter ?count .} )
-            OPTIONAL { ?song nao:hasTag ?tag .
-                       FILTER (?tag = nao:predefined-tag-favorite) }
-        } ORDER BY DESC(?lastPlayed) LIMIT 50
+            FILTER (?lastPlayed > '%(compare_date)s'^^xsd:dateTime)
+        }
         """.replace('\n', ' ').strip() % {
             "media_type": int(Grl.MediaType.AUDIO),
             'compare_date': compare_date,
@@ -1062,6 +1078,7 @@ class RecentlyAdded(SmartPlaylist):
         self.props.tag_text = "RECENTLY_ADDED"
         # TRANSLATORS: this is a playlist name
         self._title = _("Recently Added")
+        self.props.icon_name = "list-add-symbolic"
 
         sparql_midnight_dateTime_format = "%Y-%m-%dT00:00:00Z"
         days_difference = 7
@@ -1124,6 +1141,7 @@ class Favorites(SmartPlaylist):
         self.props.tag_text = "FAVORITES"
         # TRANSLATORS: this is a playlist name
         self._title = _("Favorite Songs")
+        self.props.icon_name = "starred-symbolic"
         self.props.query = """
             SELECT
                 %(media_type)s AS ?type
